@@ -7,9 +7,9 @@ import cachetools
 import requests
 
 from robotoff import settings
+from robotoff.types import JSONType
 from robotoff.utils import get_logger, http_session, load_json
 from robotoff.utils.text import get_tag
-from robotoff.utils.types import JSONType
 
 try:
     import networkx
@@ -100,6 +100,10 @@ class TaxonomyNode:
         if lang in self.names:
             return self.names[lang]
 
+        if "xx" in self.names:
+            # Return international name if it exists
+            return self.names["xx"]
+
         return self.id
 
     def get_synonyms(self, lang: str) -> list[str]:
